Package: vecvec
Title: Construct Mixed Type Data Structures with Vectors of Vectors
Version: 1.1.0
Authors@R: c(
      person(
        given = "Mitchell",
        family = "O'Hara-Wild",
        role = c("aut", "cre"),
        email = "mail@mitchelloharawild.com",
        comment = c(ORCID = "0000-0001-6729-7695")
      ))
Description: Mixed type vectors are useful for combining semantically similar 
  classes. Some examples of semantically related classes include time across 
  different granularities (e.g. daily, monthly, annual) and probability 
  distributions (e.g. Normal, Uniform, Poisson). These groups of vector types 
  typically share common statistical operations which vary in results with the 
  attributes of each vector. The 'vecvec' data structure facilitates efficient 
  storage and computation across multiple vectors within the same object.
License: MIT + file LICENSE
Imports: rlang, S7, stats, vctrs
Suggests: lifecycle, testthat (>= 3.0.0)
Config/testthat/edition: 3
Encoding: UTF-8
URL: https://pkg.mitchelloharawild.com/vecvec/,
        https://github.com/mitchelloharawild/vecvec
BugReports: https://github.com/mitchelloharawild/vecvec/issues
Config/roxygen2/version: 7.3.3.9000
NeedsCompilation: no
Packaged: 2026-06-03 19:17:36 UTC; mitchell
Author: Mitchell O'Hara-Wild [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6729-7695>)
Maintainer: Mitchell O'Hara-Wild <mail@mitchelloharawild.com>
Repository: CRAN
Date/Publication: 2026-06-04 05:10:03 UTC
Built: R 4.6.0; ; 2026-06-04 09:26:51 UTC; unix
