# Generated by roxygen2: do not edit by hand

S3method(Complex,"vecvec::vecvec")
S3method(Math,"vecvec::vecvec")
S3method(Ops,"vecvec::vecvec")
S3method(Summary,"vecvec::vecvec")
S3method(`[<-`,"vecvec::vecvec")
S3method(`[[<-`,"vecvec::vecvec")
S3method(`dim<-`,"vecvec::vecvec")
S3method(`dimnames<-`,"vecvec::vecvec")
S3method(`is.na<-`,"vecvec::vecvec")
S3method(`length<-`,"vecvec::vecvec")
S3method(as.double,"vecvec::vecvec")
S3method(as.vector,"vecvec::vecvec")
S3method(vec_cast,"vecvec::vecvec")
S3method(vec_ptype2,"vecvec::vecvec.vecvec::vecvec")
export(class_vecvec)
export(cor)
export(cov)
export(is_vecvec)
export(unvecvec)
export(var)
export(vecvec)
export(vecvec_apply)
export(vecvec_apply_fn)
export(vecvec_register)
import(S7)
import(vctrs)
importFrom(rlang,expr)
importFrom(rlang,exprs)
importFrom(rlang,quo)
importFrom(rlang,quos)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(stats,median)
