% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vecvec-package.R
\docType{package}
\name{vecvec-package}
\alias{vecvec-package}
\title{vecvec: Construct Mixed Type Data Structures with Vectors of Vectors}
\description{
\if{html}{\figure{logo.svg}{options: style='float: right' alt='logo' width='120'}}

Mixed type vectors are useful for combining semantically similar classes. Some examples of semantically related classes include time across different granularities (e.g. daily, monthly, annual) and probability distributions (e.g. Normal, Uniform, Poisson). These groups of vector types typically share common statistical operations which vary in results with the attributes of each vector. The 'vecvec' data structure facilitates efficient storage and computation across multiple vectors within the same object.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://pkg.mitchelloharawild.com/vecvec/}
  \item \url{https://github.com/mitchelloharawild/vecvec}
  \item Report bugs at \url{https://github.com/mitchelloharawild/vecvec/issues}
}

}
\author{
\strong{Maintainer}: Mitchell O'Hara-Wild \email{mail@mitchelloharawild.com} (\href{https://orcid.org/0000-0001-6729-7695}{ORCID})

Authors:
\itemize{
  \item Mitchell O'Hara-Wild \email{mail@mitchelloharawild.com} (\href{https://orcid.org/0000-0001-6729-7695}{ORCID})
}

}
\keyword{internal}
